package sun.rmi.server;

import sun.security.action.GetPropertyAction;
import java.security.AccessController;
import sun.rmi.runtime.NewThreadAction;
import java.util.Date;
import java.io.IOException;
import java.io.OutputStream;
import java.io.InputStream;
import java.io.PrintWriter;
import java.io.ByteArrayOutputStream;

class PipeWriter implements Runnable
{
    private ByteArrayOutputStream bufOut;
    private int cLast;
    private byte[] currSep;
    private PrintWriter out;
    private InputStream in;
    private String pipeString;
    private String execString;
    private static String lineSeparator;
    private static int lineSeparatorLength;
    private static int numExecs;
    
    private PipeWriter(InputStream in, OutputStream out, String str, int i) {
        this.in = in;
        this.out = new PrintWriter(out);
        this.bufOut = new ByteArrayOutputStream();
        this.currSep = new byte[PipeWriter.lineSeparatorLength];
        this.execString = ":ExecGroup-" + Integer.toString(i) + 58 + str + 58;
    }
    
    @Override
    public void run() {
        byte[] b = new byte[256];
        try {
            int read;
            while ((read = this.in.read(b)) != -1) {
                this.write(b, 0, read);
            }
            String string = this.bufOut.toString();
            this.bufOut.reset();
            if (string.length() > 0) {
                this.out.println(this.createAnnotation() + string);
                this.out.flush();
            }
        }
        catch (IOException ex) {}
    }
    
    private void write(byte[] array, int n, int index) throws IOException {
        if (index < 0) {
            throw new ArrayIndexOutOfBoundsException(index);
        }
        for (int i = 0; i < index; ++i) {
            this.write(array[n + i]);
        }
        return;
    }
    
    private void write(byte b) throws IOException {
        int i;
        for (i = 1; i < this.currSep.length; ++i) {
            this.currSep[i - 1] = this.currSep[i];
        }
        this.currSep[i - 1] = b;
        this.bufOut.write((int)b);
        if (this.cLast >= PipeWriter.lineSeparatorLength - 1 && PipeWriter.lineSeparator.equals(new String(this.currSep))) {
            this.cLast = 0;
            this.out.print(this.createAnnotation() + this.bufOut.toString());
            this.out.flush();
            this.bufOut.reset();
            if (this.out.checkError()) {
                throw new IOException("PipeWriter: IO Exception when writing to output stream.");
            }
        }
        else {
            ++this.cLast;
        }
    }
    
    private String createAnnotation() {
        return new Date().toString() + this.execString;
    }
    
    static void plugTogetherPair(InputStream inputStream, OutputStream outputStream, InputStream inputStream2, OutputStream outputStream2) {
        int numExec = getNumExec();
        Thread thread = (Thread)AccessController.doPrivileged(new NewThreadAction(new PipeWriter(inputStream, outputStream, "out", numExec), "out", true));
        Thread thread2 = (Thread)AccessController.doPrivileged(new NewThreadAction(new PipeWriter(inputStream2, outputStream2, "err", numExec), "err", true));
        thread.start();
        thread2.start();
    }
    
    private static synchronized int getNumExec() {
        return PipeWriter.numExecs++;
    }
    
    static {
        PipeWriter.numExecs = 0;
        PipeWriter.lineSeparator = (String)AccessController.doPrivileged(new GetPropertyAction("line.separator"));
        PipeWriter.lineSeparatorLength = PipeWriter.lineSeparator.length();
    }
}
